@@ -72,7 +72,7 @@ static struct plugin **plugint=&pluginh;
 
 void addcl(int ncl,struct comlist *cl)
 {
-	register int i;
+	int i;
 	for (i=0;i<ncl;i++,cl++) {
 		cl->next=NULL;
 		(*clt)=cl;
@@ -82,9 +82,9 @@ void addcl(int ncl,struct comlist *cl)
 
 void delcl(int ncl,struct comlist *cl)
 {
-	register int i;
+	int i;
 	for (i=0;i<ncl;i++,cl++) {
-		register struct comlist **p=&clh;
+		struct comlist **p=&clh;
 		while (*p != NULL) {
 			if (*p == cl) 
 				*p=cl->next;
@@ -109,9 +109,9 @@ void adddbgcl(int ncl,struct dbgcl *cl)
 
 void deldbgcl(int ncl,struct dbgcl *cl)
 {
-	register int i;
+	int i;
 	for (i=0;i<ncl;i++,cl++) {
-		register struct dbgcl **p=&dbgclh;
+		struct dbgcl **p=&dbgclh;
 		while (*p != NULL) {
 			if (*p == cl) {
 				if (cl->fds) free(cl->fds);
@@ -136,7 +136,7 @@ void addplugin(struct plugin *cl)
 
 void delplugin(struct plugin *cl)
 {
-	register struct plugin **p=&pluginh;
+	struct plugin **p=&pluginh;
 	while (*p != NULL) {
 		if (*p == cl)
 			*p=cl->next;
@@ -381,7 +381,7 @@ void mgmtnewfd(int new)
 		return;
 	}
 
-	add_fd(new,mgmt_data,-1);
+	add_fd(new,mgmt_data,NULL);
 	EVENTOUT(MGMTPORTNEW,new);
 	snprintf(buf,MAXCMD,header,PACKAGE_VERSION);
 	write(new,buf,strlen(buf));
@@ -392,7 +392,7 @@ void mgmtnewfd(int new)
 static int debugdel(int fd,char *arg);
 #endif
 static char *EOS="9999 END OF SESSION";
-static void handle_input(unsigned char type,int fd,int revents,int *unused)
+static void handle_io(unsigned char type,int fd,int revents,void *unused)
 {
 	char buf[MAXCMD];
 	if (type != mgmt_ctl) {
@@ -451,7 +451,7 @@ static void handle_input(unsigned char type,int fd,int revents,int *unused)
 			return;
 		}
 
-		add_fd(new,mgmt_data,-1);
+		add_fd(new,mgmt_data,NULL);
 		EVENTOUT(MGMTPORTNEW,new);
 		snprintf(buf,MAXCMD,header,PACKAGE_VERSION);
 		write(new,buf,strlen(buf));
@@ -462,9 +462,9 @@ static void handle_input(unsigned char type,int fd,int revents,int *unused)
 static void save_pidfile()
 {
 	if(pidfile[0] != '/')
-		strncat(pidfile_path, pidfile, PATH_MAX - strlen(pidfile_path));
+		strncat(pidfile_path, pidfile, PATH_MAX - strlen(pidfile_path) - 1);
 	else
-		strcpy(pidfile_path, pidfile);
+		strncpy(pidfile_path, pidfile, PATH_MAX - 1);
 
 	int fd = open(pidfile_path,
 			O_WRONLY | O_CREAT | O_EXCL,
@@ -489,7 +489,7 @@ static void save_pidfile()
 	fclose(f);
 }
 
-static void cleanup(unsigned char type,int fd,int arg)
+static void cleanup(unsigned char type,int fd,void *unused)
 {
 	if (fd < 0) {
 		if((pidfile != NULL) && unlink(pidfile_path) < 0) {
@@ -567,15 +567,15 @@ static void init(void)
 	}
 	/* add stdin (if tty), connect and data fds to the set of fds we wait for
 	 *    * input */
-	if(isatty(0) && !daemonize)
+	if(!daemonize)
 	{
 		console_type=add_type(&swmi,0);
-		add_fd(0,console_type,-1);
+		add_fd(0,console_type,NULL);
 	}
 
 	/* saves current path in pidfile_path, because otherwise with daemonize() we
 	 *    * forget it */
-	if(getcwd(pidfile_path, PATH_MAX-1) == NULL) {
+	if(getcwd(pidfile_path, PATH_MAX-2) == NULL) {
 		printlog(LOG_ERR, "getcwd: %s", strerror(errno));
 		exit(1);
 	}
@@ -623,7 +623,7 @@ static void init(void)
 		}
 		mgmt_ctl=add_type(&swmi,0);
 		mgmt_data=add_type(&swmi,0);
-		add_fd(mgmtconnfd,mgmt_ctl,-1);
+		add_fd(mgmtconnfd,mgmt_ctl,NULL);
 	}
 }
 
@@ -867,7 +867,7 @@ void start_consmgmt(void)
 	swmi.usage=usage;
 	swmi.parseopt=parseopt;
 	swmi.init=init;
-	swmi.handle_input=handle_input;
+	swmi.handle_io=handle_io;
 	swmi.cleanup=cleanup;
 	ADDCL(cl);
 #ifdef DEBUGOPT
