@@ -27,11 +27,6 @@
 #include "../vde_switch/switch.h"
 #include "consmgmt.h"
 #include "datasock.h"
-#undef VDE_PQ
-#undef OPTPOLL
-#ifdef VDE_PQ
-#include <packetq.h>
-#endif
 
 time_t starting_time;
 static struct swmodule *swmh;
@@ -84,7 +79,7 @@ void del_swm(struct swmodule *old)
 /* FD MGMT */
 struct pollplus {
 	unsigned char type;
-	int arg;
+	void *private_data;
 	time_t timestamp;
 };
 
@@ -107,7 +102,7 @@ static int maxtypes;
 
 unsigned char add_type(struct swmodule *mgr,int prio)
 {
-	register int i;
+	int i;
 	if(ntypes==maxtypes) {
 		maxtypes = maxtypes ? 2 * maxtypes : 8;
 		if (maxtypes > PRIOFLAG) {
@@ -136,7 +131,7 @@ void del_type(unsigned char type)
 	ntypes--;
 }
 
-void add_fd(int fd,unsigned char type,int arg)
+void add_fd(int fd,unsigned char type,void *private_data)
 {
 	struct pollfd *p;
 	int index;
@@ -167,20 +162,20 @@ void add_fd(int fd,unsigned char type,int arg)
 	p->fd = fd;
 	p->events = POLLIN | POLLHUP;
 	fdpp[index]->type=type;
-	fdpp[index]->arg=arg;
+	fdpp[index]->private_data=private_data;
 	nfds++;
 }
 
 static void file_cleanup(void)
 {
-	register int i;
+	int i;
 	for(i = 0; i < nfds; i++)
-		TYPE2MGR(fdpp[i]->type)->cleanup(fdpp[i]->type,fds[i].fd,fdpp[i]->arg);
+		TYPE2MGR(fdpp[i]->type)->cleanup(fdpp[i]->type,fds[i].fd,fdpp[i]->private_data);
 }
 
 void remove_fd(int fd)
 {
-	register int i;
+	int i;
 
 	for(i = 0; i < nfds; i++){
 		if(fds[i].fd == fd) break;
@@ -189,7 +184,7 @@ void remove_fd(int fd)
 		printlog(LOG_WARNING,"remove_fd : Couldn't find descriptor %d", fd);
 	} else {
 		struct pollplus *old=fdpp[i];
-		TYPE2MGR(fdpp[i]->type)->cleanup(fdpp[i]->type,fds[i].fd,fdpp[i]->arg);
+		TYPE2MGR(fdpp[i]->type)->cleanup(fdpp[i]->type,fds[i].fd,fdpp[i]->private_data);
 		if (ISPRIO(fdpp[i]->type)) nprio--;
 		memmove(&fds[i], &fds[i + 1], (maxfds - i - 1) * sizeof(struct pollfd));
 		memmove(&fdpp[i], &fdpp[i + 1], (maxfds - i - 1) * sizeof(struct pollplus *));
@@ -201,13 +196,9 @@ void remove_fd(int fd)
 static void main_loop()
 {
 	time_t now;
-	register int n,i;
+	int n,i;
 	while(1) {
-#ifdef VDE_PQ
-		n=poll(fds,nfds,packetq_timeout);
-#else
 		n=poll(fds,nfds,-1);
-#endif
 		now=time(NULL);
 		if(n < 0){ 
 			if(errno != EINTR)
@@ -215,33 +206,15 @@ static void main_loop()
 		} else {
 			for(i = 0; /*i < nfds &&*/ n>0; i++){
 				if(fds[i].revents != 0) {
-					register int prenfds=nfds;
+					int prenfds=nfds;
 					n--;
 					fdpp[i]->timestamp=now;
-					TYPE2MGR(fdpp[i]->type)->handle_input(fdpp[i]->type,fds[i].fd,fds[i].revents,&(fdpp[i]->arg));
+					TYPE2MGR(fdpp[i]->type)->handle_io(fdpp[i]->type,fds[i].fd,fds[i].revents,fdpp[i]->private_data);
 					if (nfds!=prenfds) /* the current fd has been deleted */
 						break; /* PERFORMANCE it is faster returning to poll */
 				}	
 /* optimization: most used descriptors migrate to the head of the poll array */
-#ifdef OPTPOLL
-				else
-				{
-					if (i < nfds && i > 0 && i != nprio) {
-						register int i_1=i-1;
-						if (fdpp[i]->timestamp > fdpp[i_1]->timestamp) {
-							struct pollfd tfds;
-							struct pollplus *tfdpp;
-							tfds=fds[i];fds[i]=fds[i_1];fds[i_1]=tfds;
-							tfdpp=fdpp[i];fdpp[i]=fdpp[i_1];fdpp[i_1]=tfdpp;
-						}
-					}
-				}
-#endif
 			}
-#ifdef VDE_PQ
-			if (packetq_timeout > 0)
-				packetq_try();
-#endif
 		}
 	}
 }
@@ -285,7 +258,7 @@ static void version(void)
 
 static struct option *optcpy(struct option *tgt, struct option *src, int n, int tag)
 {
-	register int i;
+	int i;
 	memcpy(tgt,src,sizeof(struct option) * n);
 	for (i=0;i<n;i++) {
 		tgt[i].val=(tgt[i].val & 0xffff) | tag << 16;
@@ -330,7 +303,7 @@ static void parse_args(int argc, char **argv)
 	if (long_options == NULL || optstring==NULL)
 		exit(2);
 	{ /* fill-in the long_options fields */
-		register int i;
+		int i;
 		char *os=optstring;
 		char last=0;
 		struct option *opp=long_options;
@@ -402,7 +375,7 @@ static void cleanup(void)
 	file_cleanup();
 	for(swmp=swmh;swmp != NULL;swmp=swmp->next)
 		if (swmp->cleanup != NULL)
-			swmp->cleanup(0,-1,-1);
+			swmp->cleanup(0,-1,NULL);
 }
 
 static void sig_handler(int sig)
@@ -410,7 +383,10 @@ static void sig_handler(int sig)
 	printlog(LOG_ERR,"Caught signal %d, cleaning up and exiting", sig);
 	cleanup();
 	signal(sig, SIG_DFL);
-	kill(getpid(), sig);
+	if (sig == SIGTERM)
+		_exit(0);
+	else
+		kill(getpid(), sig);
 }
 
 static void setsighandlers()
