@@ -12,6 +12,7 @@
 #include <unistd.h>
 #include <syslog.h>
 #include <stdlib.h>
+#include <signal.h>
 #include <grp.h>
 #include <libgen.h>
 #include <sys/types.h>
@@ -64,9 +65,11 @@ static struct dbgcl *dbgclh=NULL;
 static struct dbgcl **dbgclt=&dbgclh;
 #define MGMTPORTNEW (dl) 
 #define MGMTPORTDEL (dl+1) 
+#define MGMTSIGHUP (dl+2) 
 static struct dbgcl dl[]= {
 	{"mgmt/+",NULL,D_MGMT|D_PLUS},
-	{"mgmt/-",NULL,D_MGMT|D_MINUS}
+	{"mgmt/-",NULL,D_MGMT|D_MINUS},
+	{"sig/hup",NULL,D_SIG|D_HUP}
 };
 #endif
 #ifdef VDEPLUGIN
@@ -140,7 +143,7 @@ void addplugin(struct plugin *cl)
 
 void delplugin(struct plugin *cl)
 {
-	register struct plugin **p=&pluginh;
+	register struct plugin **p=plugint=&pluginh;
 	while (*p != NULL) {
 		if (*p == cl)
 			*p=cl->next;
@@ -307,8 +310,11 @@ static int handle_cmd(int type,int fd,char *inbuf)
 				}
 			}
 		}
-		if (rv >=0)
+		if (rv == 0) {
+			printoutc(f,"1000 Success");
+		} else if (rv > 0) {
 			printoutc(f,"1%03d %s",rv,strerror(rv));
+		}
 		fclose(f);
 		if (fd >= 0)
 			write(fd,outbuf,outbufsize);
@@ -364,7 +370,7 @@ void mgmtnewfd(int new)
 		return;
 	}
 
-	add_fd(new,mgmt_data,-1);
+	add_fd(new,mgmt_data,NULL);
 	EVENTOUT(MGMTPORTNEW,new);
 	snprintf(buf,MAXCMD,header,PACKAGE_VERSION);
 	write(new,buf,strlen(buf));
@@ -375,7 +381,7 @@ void mgmtnewfd(int new)
 static int debugdel(int fd,char *arg);
 #endif
 static char *EOS="9999 END OF SESSION";
-static void handle_input(unsigned char type,int fd,int revents,int *unused)
+static void handle_io(unsigned char type,int fd,int revents,void *private_data)
 {
 	char buf[MAXCMD];
 	if (type != mgmt_ctl) {
@@ -385,9 +391,10 @@ static void handle_input(unsigned char type,int fd,int revents,int *unused)
 			n = read(fd, buf, sizeof(buf));
 			if(n < 0){
 				printlog(LOG_WARNING,"Reading from mgmt %s",strerror(errno));
+				return;
 			}
 		}
-		if (n==0) { /*EOF*/
+		if (n==0) { /*EOF || POLLHUP*/
 			if (type == console_type) {
 				printlog(LOG_WARNING,"EOF on stdin, cleaning up and exiting");
 				exit(0);
@@ -434,7 +441,7 @@ static void handle_input(unsigned char type,int fd,int revents,int *unused)
 			return;
 		}
 
-		add_fd(new,mgmt_data,-1);
+		add_fd(new,mgmt_data,NULL);
 		EVENTOUT(MGMTPORTNEW,new);
 		snprintf(buf,MAXCMD,header,PACKAGE_VERSION);
 		write(new,buf,strlen(buf));
@@ -472,7 +479,7 @@ static void save_pidfile()
 	fclose(f);
 }
 
-static void cleanup(unsigned char type,int fd,int arg)
+static void cleanup(unsigned char type,int fd,void *private_data)
 {
 	if (fd < 0) {
 		if((pidfile != NULL) && unlink(pidfile_path) < 0) {
@@ -566,7 +573,7 @@ static void init(void)
 	if(isatty(0) && !daemonize)
 	{
 		console_type=add_type(&swmi,0);
-		add_fd(0,console_type,-1);
+		add_fd(0,console_type,NULL);
 	}
 
 	/* saves current path in pidfile_path, because otherwise with daemonize() we
@@ -619,7 +626,7 @@ static void init(void)
 		}
 		mgmt_ctl=add_type(&swmi,0);
 		mgmt_data=add_type(&swmi,0);
-		add_fd(mgmtconnfd,mgmt_ctl,-1);
+		add_fd(mgmtconnfd,mgmt_ctl,NULL);
 	}
 }
 
@@ -642,7 +649,6 @@ static int showinfo(FILE *fd)
 			qtime());
 	if (mgmt_socket)
 		printoutc(fd,"mgmt %s perm 0%03o",mgmt_socket,mgmt_mode);
-	printoutc(fd,"unsent_pktq_len %d",packetq_count());
 	return 0;
 }
 
@@ -798,11 +804,76 @@ static int pluginlist(FILE *f,char *arg)
 	return rv;
 }
 
+/* This will be prefixed with getent("$HOME") */
+#define USER_PLUGINS_DIR "/.vde2/plugins"
+
+#ifndef MAX
+# define MAX(a, b) ((a) > (b) ? (a) : (b))
+#endif
+/*
+ * Try to dlopen a plugin trying different names and locations:
+ * (code from view-os by Gardenghi)
+ * 
+ * 1) dlopen(modname)
+ * 2) dlopen(modname.so)
+ * 3) dlopen(user_umview_plugin_directory/modname)
+ * 4) dlopen(user_umview_plugin_directory/modname.so)
+ * 5) dlopen(global_umview_plugin_directory/modname)
+ * 6) dlopen(global_umview_plugin_directory/modname.so)
+ *
+ */
+
+#define TRY_DLOPEN(fmt...) \
+{ \
+	snprintf(testpath, tplen, fmt); \
+	if ((handle = dlopen(testpath, flag))) \
+	{ \
+		free(testpath); \
+		return handle; \
+	} \
+}
+
+void *plugin_dlopen(const char *modname, int flag)
+{
+	void *handle;
+	char *testpath;
+	int tplen;
+	char *homedir = getenv("HOME");
+
+	if (!modname)
+		return NULL;
+
+	if ((handle = dlopen(modname, flag)))
+		return handle;
+
+	/* If there is no home directory, use CWD */
+	if (!homedir)
+		homedir = ".";
+
+	tplen = strlen(modname) +
+		strlen(MODULES_EXT) + 2 + // + 1 is for a '/' and + 1 for \0
+		MAX(strlen(PLUGINS_DIR),
+				strlen(homedir) + strlen(USER_PLUGINS_DIR));
+
+	  testpath = malloc(tplen);
+
+		TRY_DLOPEN("%s%s", modname, MODULES_EXT);
+		TRY_DLOPEN("%s%s/%s", homedir, USER_PLUGINS_DIR, modname);
+		TRY_DLOPEN("%s%s/%s%s", homedir, USER_PLUGINS_DIR, modname, MODULES_EXT);
+		TRY_DLOPEN("%s%s", PLUGINS_DIR, modname);
+		TRY_DLOPEN("%s/%s%s", PLUGINS_DIR, modname, MODULES_EXT);
+
+		free(testpath);
+		return NULL;
+}
+
+
+
 static int pluginadd(char *arg) {
 	void *handle;
 	struct plugin *p;
 	int rv=ENOENT;
-	if ((handle=dlopen(arg,RTLD_LAZY)) != NULL) {
+	if ((handle=plugin_dlopen(arg,RTLD_LAZY)) != NULL) {
 		if ((p=(struct plugin *) dlsym(handle,"vde_plugin_data")) != NULL) {
 			if (p->handle != NULL) { /* this dyn library is already loaded*/
 				dlclose(handle);
@@ -858,6 +929,11 @@ static struct comlist cl[]={
 #endif
 };
 
+static void sighupmgmt(int signo)
+{
+	EVENTOUT(MGMTSIGHUP, signo);
+}
+
 void start_consmgmt(void)
 {
 	swmi.swmname="console-mgmt";
@@ -866,11 +942,14 @@ void start_consmgmt(void)
 	swmi.usage=usage;
 	swmi.parseopt=parseopt;
 	swmi.init=init;
-	swmi.handle_input=handle_input;
+	swmi.handle_io=handle_io;
 	swmi.cleanup=cleanup;
 	ADDCL(cl);
 #ifdef DEBUGOPT
 	ADDDBGCL(dl);
 #endif
 	add_swm(&swmi);
+#ifdef DEBUGOPT
+	signal(SIGHUP,sighupmgmt);
+#endif
 }
