@@ -46,39 +46,51 @@ struct hash_entry {
 	struct hash_entry **prev;
 	time_t last_seen;
 	int port;
-	unsigned char dst[ETH_ALEN+2];
+	u_int64_t dst;
 };
 
 static struct hash_entry **h;
 
-static int calc_hash(unsigned char *src)
+static int calc_hash(u_int64_t src)
 {
-	register int x= (*(u_int32_t *) &src[0]);
-	register int y= (*(u_int32_t *) &src[4]);
-	x = x * 0x03050709 + y * 0x0b0d1113;
-	x = (x ^ x >> 12 ^ x >> 8 ^ x >> 4) & hash_mask;
-	/*printf("HASH %02x:%02x:%02x:%02x:%02x:%02x V%d -> %d\n", src[0], src[1], src[2], src[3], src[4], src[5],(src[6]>>8)+src[7],x);*/
-	return x; 
+	src ^= src >> 33;
+	src *= 0xff51afd7ed558ccd;
+	src ^= src >> 33;
+	src *= 0xc4ceb9fe1a85ec53;
+	return src & hash_mask;
 }
 
+#if BYTE_ORDER == LITTLE_ENDIAN
+#define EMAC2MAC6(X) \
+	(u_int)((X)&0xff), (u_int)(((X)>>8)&0xff), (u_int)(((X)>>16)&0xff), \
+(u_int)(((X)>>24)&0xff), (u_int)(((X)>>32)&0xff), (u_int)(((X)>>40)&0xff)
+#elif BYTE_ORDER == BIG_ENDIAN
+#define EMAC2MAC6(X) \
+	(u_int)(((X)>>24)&0xff), (u_int)(((X)>>16)&0xff), (u_int)(((X)>>8)&0xff), \
+  (u_int)((X)&0xff), (u_int)(((X)>>40)&0xff), (u_int)(((X)>>32)&0xff)
+#else
+#error Unknown Endianess
+#endif
+
+#define EMAC2VLAN(X) ((u_int16_t) ((X)>>48))
+#define EMAC2VLAN2(X) ((u_int) (((X)>>48) &0xff)), ((u_int) (((X)>>56) &0xff))
+
 #define find_entry(MAC) \
 	({struct hash_entry *e; \
 	 int k = calc_hash(MAC);\
-	 for(e = h[k]; e && memcmp(&e->dst, (MAC), ETH_ALEN+2); e = e->next)\
+	 for(e = h[k]; e && e->dst != (MAC); e = e->next)\
 	 ;\
 	 e; })
 
-#define extmac(EMAC,MAC,VLAN) \
-	({ memcpy(EMAC,(MAC),ETH_ALEN); \
-	   *((u_int16_t *)(EMAC+ETH_ALEN))=(u_int16_t) VLAN; \
-	   EMAC; })
+
+#define extmac(MAC,VLAN) \
+	    ((*(u_int32_t *) &((MAC)[0])) + ((u_int64_t) ((*(u_int16_t *) &((MAC)[4]))+ ((u_int64_t) (VLAN) << 16)) << 32))
 
 /* looks in global hash table 'h' for given address, and return associated
  * port */
 int find_in_hash(unsigned char *dst,int vlan)
 {
-	unsigned char edst[ETH_ALEN+2];
-	struct hash_entry *e = find_entry(extmac(edst,dst,vlan));
+	struct hash_entry *e = find_entry(extmac(dst,vlan));
 	if(e == NULL) return -1;
 	return(e->port);
 }
@@ -86,12 +98,12 @@ int find_in_hash(unsigned char *dst,int vlan)
 
 int find_in_hash_update(unsigned char *src,int vlan,int port)
 {
-	unsigned char esrc[ETH_ALEN+2];
 	struct hash_entry *e;
-	int k = calc_hash(extmac(esrc,src,vlan));
+	u_int64_t esrc=extmac(src,vlan);
+	int k = calc_hash(esrc);
 	int oldport;
 	time_t now;
-	for(e = h[k]; e && memcmp(&e->dst, esrc, ETH_ALEN+2); e = e->next)
+	for(e = h[k]; e && e->dst != esrc; e = e->next)
 		;
 	if(e == NULL) {
 		e = (struct hash_entry *) malloc(sizeof(*e));
@@ -101,9 +113,9 @@ int find_in_hash_update(unsigned char *src,int vlan,int port)
 		}
 
 		DBGOUT(DBGHASHNEW,"%02x:%02x:%02x:%02x:%02x:%02x VLAN %02x:%02x Port %d",
-				     esrc[0], esrc[1], esrc[2], esrc[3], esrc[4], esrc[5], esrc[6], esrc[7], port);
+				EMAC2MAC6(esrc), EMAC2VLAN2(esrc), port);
 		EVENTOUT(DBGHASHNEW,esrc);
-		memcpy(&e->dst, esrc, ETH_ALEN+2);
+		e->dst = esrc;
 		if(h[k] != NULL) h[k]->prev = &(e->next);
 		e->next = h[k];
 		e->prev = &(h[k]);
@@ -125,7 +137,7 @@ int find_in_hash_update(unsigned char *src,int vlan,int port)
 
 #define delete_hash_entry(OLD) \
 	({ \
-	 DBGOUT(DBGHASHDEL,"%02x:%02x:%02x:%02x:%02x:%02x VLAN %02x:%02x Port %d", OLD->dst[0], OLD->dst[1], OLD->dst[2], OLD->dst[3], OLD->dst[4], OLD->dst[5], OLD->dst[6], OLD->dst[7], OLD->port);\
+	 DBGOUT(DBGHASHDEL,"%02x:%02x:%02x:%02x:%02x:%02x VLAN %02x:%02x Port %d", EMAC2MAC6(OLD->dst), EMAC2VLAN2(OLD->dst), OLD->port); \
 	 EVENTOUT(DBGHASHDEL,OLD->dst);\
 	 *((OLD)->prev)=(OLD)->next; \
 	 if((OLD)->next != NULL) (OLD)->next->prev = (OLD)->prev; \
@@ -135,8 +147,7 @@ int find_in_hash_update(unsigned char *src,int vlan,int port)
 
 void delete_hash(unsigned char *dst,int vlan)
 {
-	unsigned char edst[ETH_ALEN+2];
-	struct hash_entry *old = find_entry(extmac(edst,dst,vlan));
+	struct hash_entry *old = find_entry(extmac(dst,vlan));
 
 	if(old == NULL) return;
 	qtime_csenter();
@@ -176,7 +187,7 @@ void hash_delete_port (int port)
 static void delete_vlan_iterator (struct hash_entry *e, void *arg)
 {
 	int *vlan=(int *)arg;
-	if (*((u_int16_t *)(e->dst+ETH_ALEN)) == (u_int16_t)(*vlan))
+	if (EMAC2VLAN(e->dst) == (u_int16_t)(*vlan))
 		delete_hash_entry(e);
 }
 
@@ -192,7 +203,7 @@ struct vlanport {int vlan; int port;};
 static void delete_vlanport_iterator (struct hash_entry *e, void *arg)
 {
 	struct vlanport *vp=(struct vlanport *)arg;
-	if (*((u_int16_t *)(e->dst+ETH_ALEN)) == (u_int16_t)(vp->vlan) &&
+	if ((EMAC2VLAN(e->dst)) == (u_int16_t)(vp->vlan) &&
 			e->port == vp->port)
 		delete_hash_entry(e);
 }
@@ -210,7 +221,7 @@ struct vlansetofports {int vlan; bitarray setofports;};
 static void delete_vlansetofports_iterator (struct hash_entry *e, void *arg)
 {
 	struct vlansetofports *vp=(struct vlansetofports *)arg;
-	if (*((u_int16_t *)(e->dst+ETH_ALEN)) == (u_int16_t)(vp->vlan) &&
+	if ((EMAC2VLAN(e->dst)) == (u_int16_t)(vp->vlan) &&
 			ba_check(vp->setofports,e->port))
 		delete_hash_entry(e);
 }
@@ -274,8 +285,8 @@ static inline int po2round(int vx)
 	if (vx == 0)
 		return 0;
 	else {
-		register int i=0;
-		register int x=vx-1;
+		int i=0;
+		int x=vx-1;
 		while (x) { x>>=1; i++; }
 		if (vx != 1<<i)
 			printlog(LOG_WARNING,"Hash size must be a power of 2. %d rounded to %d",vx,1<<i);
@@ -329,8 +340,8 @@ int hash_get_gc_expire()
 static int find_hash(FILE *fd,char *strmac)
 {
 	int maci[ETH_ALEN];
-	unsigned char mac[ETH_ALEN];
-	unsigned char emac[ETH_ALEN+2];
+	unsigned char macv[ETH_ALEN];
+	unsigned char *mac=macv;
 	int rv=-1;
 	int vlan=0;
 	struct hash_entry *e;
@@ -341,17 +352,16 @@ static int find_hash(FILE *fd,char *strmac)
 	if (rv < 6)
 		return EINVAL;
 	else	{
-		register int i;
+		int i;
 		for (i=0;i<ETH_ALEN;i++)
 			mac[i]=maci[i];
-		e=find_entry(extmac(emac,mac,vlan));
+		e=find_entry(extmac(mac,vlan));
 		if (e==NULL)
 			return ENODEV;
 		else {
 			printoutc(fd,"Hash: %04d Addr: %02x:%02x:%02x:%02x:%02x:%02x VLAN %04d to port: %03d  "
 					"age %ld secs", calc_hash(e->dst),
-					e->dst[0], e->dst[1], e->dst[2], e->dst[3], e->dst[4], e->dst[5],
-					((e->dst[6]<<8) + e->dst[7]), e->port+1, qtime() - e->last_seen);
+				EMAC2MAC6(e->dst),EMAC2VLAN(e->dst), e->port+1, qtime() - e->last_seen);
 			return 0;
 		}
 	}
@@ -363,9 +373,7 @@ static void print_hash_entry(struct hash_entry *e, void *arg)
 	FILE *pfd=arg;
 	printoutc(pfd,"Hash: %04d Addr: %02x:%02x:%02x:%02x:%02x:%02x VLAN %04d to port: %03d  " 
 			"age %ld secs", calc_hash(e->dst),
-			e->dst[0], e->dst[1], e->dst[2], e->dst[3], e->dst[4], e->dst[5],
-			*((u_int16_t *)(e->dst+ETH_ALEN)),
-			e->port, qtime() - e->last_seen);
+			EMAC2MAC6(e->dst),EMAC2VLAN(e->dst), e->port, qtime() - e->last_seen);
 }
 
 static int print_hash(FILE *fd)
