@@ -132,7 +132,10 @@ static void sig_handler(int sig)
 	fprintf(stderr,"%s: Caught signal %d, cleaning up and exiting\n", vdeqname, sig);
 	cleanup();
 	signal(sig, SIG_DFL);
-	kill(getpid(), sig);
+	if (sig == SIGTERM)
+		_exit(0);
+	else
+		kill(getpid(), sig);
 }
 
 static void setsighandlers()
@@ -346,7 +349,7 @@ int main(int argc, char **argv)
 
     if (argc > 2 && ((strcmp(argv[1],"--mod")==0))
 	    ){
-	sscanf(argv[2],"%o",&mode);
+	sscanf(argv[2],"%o",(unsigned int *)&mode);
 	argv+=2;
 	argc-=2;
     }
