@@ -77,7 +77,7 @@ struct wirevalue {
 #define DDUP 3
 #define BAND 4
 #define SPEED 5
-#define CAPACITY 6
+#define CHANBUFSIZE 6
 #define NOISE 7
 #define MTU 8
 #define NUMVALUES 9
@@ -375,26 +375,31 @@ static int read_wirevalue(char *s, int tag)
 
 struct packpq {
 	unsigned long long when;
+	unsigned int counter;
 	int dir;
 	unsigned char *buf;
 	int size;
 };
 
 struct packpq **pqh;
-struct packpq sentinel={0,0,NULL,0};
+struct packpq sentinel={0,0,0,NULL,0};
 int npq,maxpq;
 unsigned long long maxwhen;
+unsigned int counter;
 
 #define PQCHUNK 100
 
-static int nextms()
+static unsigned long long nextms()
 {
 	if (npq>0) {
-		long long deltat;
+		unsigned long long now=0;
 		struct timeval v;
 		gettimeofday(&v,NULL);
-		deltat=pqh[1]->when-(v.tv_sec*1000000+v.tv_usec);
-		return (deltat>0)?(int)(deltat/1000):0;
+		now = (unsigned long long) v.tv_sec*1000+v.tv_usec/1000;
+		if (pqh[1]->when > now)
+			return  pqh[1]->when - now;
+		else
+			return 0; 
 	}
 	return -1;
 }
@@ -440,12 +445,16 @@ int writepacket(int dir,const unsigned char *buf,int size)
 /* packet queues are priority queues implemented on a heap.
  * enqueue time = dequeue time = O(log n) max&mean
  */
+/* the delay is evaluated in milliseconds, several packets can be
+	 scheduled at the same "when" time. Counter preserve the fifoness. */
 
 static void packet_dequeue()
 {
 	struct timeval v;
 	gettimeofday(&v,NULL);
-	unsigned long long now=v.tv_sec*1000000+v.tv_usec;
+	unsigned long long now=(unsigned long long)v.tv_sec*1000+v.tv_usec/1000; 
+	/* the next packet (min time, min counter) is in the root of
+		 the packetqueue heap */
 	while (npq>0 && pqh[1]->when <= now) {
 		struct packpq *old=pqh[npq--];
 		int k=1;
@@ -453,13 +462,25 @@ static void packet_dequeue()
 		writepacket(pqh[1]->dir,pqh[1]->buf,pqh[1]->size);
 		free(pqh[1]->buf);
 		free(pqh[1]);
+		/* rebuild the heap */
 		while (k<= npq>>1)
 		{
 			int j= k<<1;
-			if (j<npq && pqh[j]->when > pqh[j+1]->when) j++;
-			if (old->when <= pqh[j]->when) {
+			/* choose the min between pqh[2k] and pqh[2k+1] */
+			if (j<npq && 
+					(pqh[j]->when > pqh[j+1]->when ||
+					 (pqh[j]->when == pqh[j+1]->when && 
+						pqh[j]->counter > pqh[j+1]->counter)
+					)
+				 ) j++;
+			/* if old must be put here, okay else move the min up and
+				 continue the rebuilding phase */
+			if (old->when < pqh[j]->when || 
+					(old->when == pqh[j]->when &&
+					 old->counter < pqh[j]->counter)
+				 ) 
 				break;
-			} else {
+			else {
 				pqh[k]=pqh[j];k=j;
 			}
 		}
@@ -471,24 +492,22 @@ static void packet_enqueue(int dir,const unsigned char *buf,int size,int delms)
 {
 	struct timeval v;
 
-	/* CAPACITY */
-	/* when bandwidth is limited, packets exceeding capacity are discarded */
-	if (max_wirevalue(markov_current,CAPACITY,dir) > 0) {
-		double capval=compute_wirevalue(CAPACITY,dir);
-		if ((delay_bufsize[dir]+size) > capval)
-			return;
-	}
-	/* */
-
 	struct packpq *new=malloc(sizeof(struct packpq));
 	if (new==NULL) {
 		printlog(LOG_WARNING,"malloc elem %s",strerror(errno));
 		exit (1);
 	}
 	gettimeofday(&v,NULL);
-	new->when=v.tv_sec * 1000000 + v.tv_usec + delms * 1000;
-	if (new->when > maxwhen) maxwhen=new->when;
-	if (!nofifo && new->when < maxwhen) new->when=maxwhen;
+	new->when= ((unsigned long long)v.tv_sec * 1000 + v.tv_usec/1000) + delms; 
+	if (new->when > maxwhen) {
+		maxwhen=new->when;
+		counter=0;
+	}
+	if (!nofifo && new->when <= maxwhen) {
+		new->when=maxwhen;
+		counter++;
+	}
+	new->counter=counter;
 	new->dir=dir;
 	new->buf=malloc(size);
 	if (new->buf==NULL) {
@@ -513,8 +532,11 @@ static void packet_enqueue(int dir,const unsigned char *buf,int size,int delms)
 			exit (1);
 		}
 	}
-	{int k=++npq;
-		while (new->when < pqh[k>>1]->when) {
+	{
+		int k=++npq;
+		/* add the new element to the heap */
+		while (new->when < pqh[k>>1]->when ||
+				(new->when == pqh[k>>1]->when && new->counter < pqh[k>>1]->counter)) {
 			pqh[k]=pqh[k>>1];
 			k >>= 1;
 		}
@@ -571,6 +593,14 @@ void handle_packet(int dir,const unsigned char *buf,int size)
 	while (times>0) {
 		int banddelay=0;
 
+		/* CHANBUFSIZE */
+		/* when bandwidth is limited, packets exceeding channel bufsize are discarded */
+		if (max_wirevalue(markov_current,CHANBUFSIZE,dir) > 0) {
+			double capval=compute_wirevalue(CHANBUFSIZE,dir);
+			if ((delay_bufsize[dir]+size) > capval)
+				return;
+		}
+
 		/* SPEED */
 		/* speed limit, if packets arrive too fast, delay the sender */
 		if (max_wirevalue(markov_current,SPEED,dir) > 0) {
@@ -640,7 +670,7 @@ static void splitpacket(const unsigned char *buf,int size,int dir)
 	//fprintf(stderr,"%s: splitpacket rnx=%d remaining=%d size=%d\n",progname,rnx[dir],remaining[dir],size);
 	if (size==0) return;
 	if (rnx[dir]>0) {
-		register int amount=MIN(remaining[dir],size);
+		int amount=MIN(remaining[dir],size);
 		//fprintf(stderr,"%s: fragment amount %d\n",progname,amount);
 		memcpy(fragp[dir],buf,amount);
 		remaining[dir]-=amount;
@@ -783,9 +813,9 @@ static int check_open_fifos_n_plugs(struct pollfd *pfd,int *outfd,char *vdepath[
 static void save_pidfile()
 {
 	if(pidfile[0] != '/')
-		strncat(pidfile_path, pidfile, PATH_MAX - strlen(pidfile_path));
+		strncat(pidfile_path, pidfile, PATH_MAX - strlen(pidfile_path) - 1);
 	else
-		strcpy(pidfile_path, pidfile);
+		strncpy(pidfile_path, pidfile, PATH_MAX - 1);
 
 	int fd = open(pidfile_path,
 			O_WRONLY | O_CREAT | O_EXCL,
@@ -828,7 +858,10 @@ static void sig_handler(int sig)
 	/*fprintf(stderr,"Caught signal %d, cleaning up and exiting", sig);*/
 	cleanup();
 	signal(sig, SIG_DFL);
-	kill(getpid(), sig);
+	if (sig == SIGTERM)
+		_exit(0);
+	else
+		kill(getpid(), sig);
 } 
 
 static void setsighandlers()
@@ -984,9 +1017,9 @@ static int setspeed(int fd,char *s)
 	return read_wirevalue(s,SPEED);
 }
 
-static int setcapacity(int fd,char *s)
+static int setchanbufsize(int fd,char *s)
 {
-	return read_wirevalue(s,CAPACITY);
+	return read_wirevalue(s,CHANBUFSIZE);
 }
 
 static int setfifo(int fd,char *s)
@@ -1119,7 +1152,7 @@ static int help(int fd,char *s)
 	printoutc(fd, "speed        set interface speed bytes/sec");
 	printoutc(fd, "noise        set noise factor bits/Mbyte");
 	printoutc(fd, "mtu          set channel MTU (bytes)");
-	printoutc(fd, "capacity     set channel capacity (bytes)");
+	printoutc(fd, "chanbufsize  set channel buffer size (bytes)");
 	printoutc(fd, "fifo         set channel fifoness");
 	printoutc(fd, "shutdown     shut the channel down");
 	printoutc(fd, "logout       log out from this mgmt session");
@@ -1178,8 +1211,8 @@ static int showinfo(int fd,char *s)
 				min_wirevalue(node,MTU,LR),
 				min_wirevalue(node,MTU,RL));
 		printoutc(fd, "Cap.   L->R %g+%g%c   R->L %g+%g%c",
-				WIREVALUE_FIELDS(node,CAPACITY,LR),
-				WIREVALUE_FIELDS(node,CAPACITY,RL));
+				WIREVALUE_FIELDS(node,CHANBUFSIZE,LR),
+				WIREVALUE_FIELDS(node,CHANBUFSIZE,RL));
 		printoutc(fd, "Current Delay Queue size:   L->R %d      R->L %d   ",delay_bufsize[LR],delay_bufsize[RL]);
 	} else {
 		printoutc(fd, "Loss   %g+%g%c",
@@ -1198,7 +1231,7 @@ static int showinfo(int fd,char *s)
 			WIREVALUE_FIELDS(node,NOISE,0));
 		printoutc(fd, "MTU    %g", min_wirevalue(node,MTU,0));
 		printoutc(fd, "Cap.   %g+%g%c",
-			WIREVALUE_FIELDS(node,CAPACITY,0));
+			WIREVALUE_FIELDS(node,CHANBUFSIZE,0));
 		printoutc(fd, "Current Delay Queue size:   %d",delay_bufsize[0]);
 	}
 	printoutc(fd,"Fifoness %s",(nofifo == 0)?"TRUE":"FALSE");
@@ -1248,7 +1281,8 @@ static struct comlist {
 	{"bandwidth",setband, 0},
 	{"band",setband, 0},
 	{"speed",setspeed, 0},
-	{"capacity",setcapacity, 0},
+	{"chanbufsize",setchanbufsize, 0},
+	{"capacity",setchanbufsize, 0},
 	{"noise",setnoise, 0},
 	{"mtu",setmtu, 0},
 	{"fifo",setfifo, 0},
@@ -1380,7 +1414,7 @@ void usage(void)
 			"\t--dup|-D dup_percentage\n"
 			"\t--band|-b bandwidth(bytes/s)\n"
 			"\t--speed|-s interface_speed(bytes/s)\n"
-			"\t--capacity|-c delay_channel_capacity\n"
+			"\t--chanbufsize|-c channel_bufsize\n"
 			"\t--noise|-n noise_bits/megabye\n"
 			"\t--mtu|-m mtu_size\n"
 			"\t--nofifo|-N\n"
@@ -1411,6 +1445,7 @@ int main(int argc,char *argv[])
 		{"dup",1 , 0, 'D'},
 		{"band",1 , 0, 'b'},
 		{"speed",1 , 0, 's'},
+		{"chanbufsize",1 , 0, 'c'},
 		{"capacity",1 , 0, 'c'},
 		{"noise",1 , 0, 'n'},
 		{"mtu",1 , 0, 'm'},
@@ -1421,7 +1456,8 @@ int main(int argc,char *argv[])
 		{"daemon",0 , 0, DAEMONIZEARG},
 		{"pidfile", 1, 0, PIDFILEARG},
 		{"blink",1,0,LOGSOCKETARG},
-		{"blinkid",1,0,LOGIDARG}
+		{"blinkid",1,0,LOGIDARG},
+		{0,0,0,0}
 	};
 	progname=basename(argv[0]);
 	markov_resize(1);
@@ -1431,7 +1467,7 @@ int main(int argc,char *argv[])
 
 	while(1) {
 		int c;
-		c = GETOPT_LONG (argc, argv, "hnl:d:M:D:m:b:s:c:v:L:f:",
+		c = GETOPT_LONG (argc, argv, "hNl:n:d:M:D:m:b:s:c:v:L:f:",
 				long_options, &option_index);
 		if (c<0)
 			break;
@@ -1467,7 +1503,7 @@ int main(int argc,char *argv[])
 				read_wirevalue(optarg,SPEED);
 				break;
 			case 'c':
-				read_wirevalue(optarg,CAPACITY);
+				read_wirevalue(optarg,CHANBUFSIZE);
 				break;
 			case 'M':
 				mgmt=strdup(optarg);
@@ -1571,7 +1607,7 @@ int main(int argc,char *argv[])
 
 	/* saves current path in pidfile_path, because otherwise with daemonize() we
 	 * forget it */
-	if(getcwd(pidfile_path, PATH_MAX-1) == NULL) {
+	if(getcwd(pidfile_path, PATH_MAX-2) == NULL) {
 		printlog(LOG_ERR, "getcwd: %s", strerror(errno));
 		exit(1);
 	}
@@ -1596,7 +1632,7 @@ int main(int argc,char *argv[])
 
 	initrand();
 	while(1) {
-		int delay=nextms();
+		unsigned long long delay=nextms();
 		int markovdelay=markovms();
 		if (markovdelay >= 0 &&
 				(markovdelay < delay || delay < 0)) delay=markovdelay;
@@ -1643,7 +1679,7 @@ int main(int argc,char *argv[])
 			int mgmtfdstart=consoleindex;
 			if (mgmtindex >= 0) mgmtfdstart=mgmtindex+1;
 			if (mgmtfdstart >= 0 && npfd > mgmtfdstart) {
-				register int i;
+				int i;
 				for (i=mgmtfdstart;i<npfd;i++) {
 					if (pfd[i].revents & POLLIN && mgmtcommand(pfd[i].fd) < 0)
 						pfd[i].revents |= POLLHUP;
